#!/usr/bin/env python

##############################################################################
##
## Copyright (C) 2021 Narda Safety Test Solutions GmbH.
## Contact: www.narda-sts.com
##
## Redistribution and use in source and binary forms,
## with or without modification, are permitted provided that the following
## conditions are met:
##
## 1.) Redistributions of source code must retain the above copyright notice,
## this list of conditions and the following disclaimer.
##
## 2.) Redistributions in binary form must reproduce the above copyright
## notice, this list of conditions and the following disclaimer in the
## documentation and/or other materials provided with the distribution.
##
## 3.) Neither the name of the copyright holder nor the names of its
## contributors may be used to endorse or promote products derived from this
## software without specific prior written permission.
##
## 4.) EXPORT CONTROL
## The Software, including technical data /cryptographic software,
## may be subject to, German, European Union and U.S. export controls and
## may be subject to import or export controls in other countries.
## The Licensee agrees to strictly comply with all applicable import and
## export regulations. He specifically agrees, that he must not disclose or
## otherwise export or re-export the Licensed Software or any part thereof
## delivered under this end user license agreement (EULA) to any country
## (including a national or resident of such country) without a valid export
## or import license. Please be aware that the Software may contain
## US-Content, therefor the Licensee represent and warrant that he is not a
## citizen of, or otherwise located within, an embargoed nation (including
## without limitation Iran, Syria, Sudan, Cuba, North Korea) and that he is
## not otherwise prohibited under the Export Laws from receiving the Software.
## All rights to Use the Software are granted on condition that such rights
## are forfeited if the Licensee fails to comply with
## the terms of this Agreement.
##
## 5.) SEVERABILITY
## Should any provision of this Agreement be or become invalid, ineffective
## or unenforceable, the remaining provisions of this Agreement shall be
## valid. The parties agree to replace the invalid, ineffective or
## unenforceable provision by a valid, effective and enforceable provision
## which best meets the commercial intention of the parties.
## The same shall apply in case of omissions.
##
## 6.) APPLICABLE LAW AND PLACE OF JURISTICATION
## This Agreement shall be constituted under the law of Germany.
## The United Nations Convention on the International Sale of Goods
## shall not apply to this Agreement. The Place of Jurisdiction for any
## dispute between the Parties shall be Tübingen (Germany).
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
## “AS IS” AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Narda Safety Test Solutions GmbH
## BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
## CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
## SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
## INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
## CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
## ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
## POSSIBILITY OF SUCH DAMAGE.
##
##############################################################################


import wx
import wx.grid as grd
import queue as Queue
from nardascripting.base.signalsharkdev import *
from nardascripting.base.usrscriptbase import *
from nardascripting.base.measthread import *
import nardascripting.base.toolbox as tb
from .netdfdlgsbase import DeviceListDlgBase
from .. import filehandling as fh


class DeviceListDlg(DeviceListDlgBase):
    def __init__(self, parent):
        """Dialog initialization"""
        super().__init__(parent)
        self.init_progress_bar(self.pgb_progress)
        self.init_dlg_buttons(self.sdbs_buttons, wx.OK)
        self.SetSize(850, 500)
        self._init_table()

    def _init_table(self):
        """Initialize the table grid"""
        # Grid
        self.grd_table.ClearGrid()
        self.grd_table.InsertCols(numCols=14)
        self.grd_table.EnableEditing(True)
        self.grd_table.EnableGridLines(True)
        self.grd_table.EnableDragGridSize(True)
        self.grd_table.SetMargins(0, 0)
        self.grd_table.SetDefaultRowSize(50)

        # Column rendering
        self.grd_table.SetColLabelAlignment(wx.ALIGN_CENTRE, wx.ALIGN_CENTRE)

        cgray = (204, 204, 204)
        attr_readonly = grd.GridCellAttr()
        attr_readonly.SetReadOnly(True)
        attr_readonly.SetBackgroundColour(cgray)

        # Columns
        col = 0
        attr_bool1 = grd.GridCellAttr()
        attr_bool1.SetReadOnly(False)
        attr_bool1.SetEditor(grd.GridCellBoolEditor())
        attr_bool1.SetRenderer(grd.GridCellBoolRenderer())
        self.grd_table.SetColSize(col, 60)
        self.grd_table.SetColLabelValue(col, u"Selected")
        self.grd_table.SetColAttr(col, attr_bool1)
        col += 1

        self.grd_table.SetColSize(col, 80)
        self.grd_table.SetColLabelValue(col, u"IP")
        self.grd_table.SetColAttr(col, attr_readonly)
        col += 1

        self.grd_table.SetColSize(col, 110)
        self.grd_table.SetColLabelValue(col, u"MAC")
        self.grd_table.SetColAttr(col, attr_readonly.Clone())
        col += 1

        self.grd_table.SetColSize(col, 110)
        self.grd_table.SetColLabelValue(col, u"PN")
        self.grd_table.SetColAttr(col, attr_readonly.Clone())
        col += 1

        self.grd_table.SetColSize(col, 110)
        self.grd_table.SetColLabelValue(col, u"SN")
        self.grd_table.SetColAttr(col, attr_readonly.Clone())
        col += 1

        attr_readonly1 = grd.GridCellAttr()
        attr_readonly1.SetReadOnly(True)
        attr_readonly1.SetRenderer(grd.GridCellAutoWrapStringRenderer())
        attr_readonly1.SetBackgroundColour(cgray)
        self.grd_table.SetColSize(col, 180)
        self.grd_table.SetColLabelValue(col, u"IDN")
        self.grd_table.SetColAttr(col, attr_readonly1)
        col += 1

        attr_str = grd.GridCellAttr()
        attr_str.SetReadOnly(False)
        attr_str.SetEditor(grd.GridCellTextEditor())
        attr_str.SetRenderer(grd.GridCellStringRenderer())
        self.grd_table.SetColSize(col, 150)
        self.grd_table.SetColLabelValue(col, u"Comment")
        self.grd_table.SetColAttr(col, attr_str)
        col += 1

        attr_unit = grd.GridCellAttr()
        attr_unit.SetReadOnly(False)
        attr_unit.SetEditor(grd.GridCellChoiceEditor(choices=LevelUnits.get_str_list()))
        attr_unit.SetRenderer(grd.GridCellStringRenderer())
        self.grd_table.SetColSize(col, 60)
        self.grd_table.SetColLabelValue(col, u"Unit")
        self.grd_table.SetColAttr(col, attr_unit)
        col += 1

        attr_float1 = grd.GridCellAttr()
        attr_float1.SetReadOnly(False)
        attr_float1.SetEditor(grd.GridCellFloatEditor(precision=1))
        attr_float1.SetRenderer(grd.GridCellFloatRenderer(precision=1))
        self.grd_table.SetColSize(col, 60)
        self.grd_table.SetColLabelValue(col, u"Atten.")
        self.grd_table.SetColAttr(col, attr_float1)
        col += 1

        attr_bool2 = grd.GridCellAttr()
        attr_bool2.SetReadOnly(False)
        attr_bool2.SetEditor(grd.GridCellBoolEditor())
        attr_bool2.SetRenderer(grd.GridCellBoolRenderer())
        self.grd_table.SetColSize(col, 60)
        self.grd_table.SetColLabelValue(col, u"Adapt\r\nAtten.")
        self.grd_table.SetColAttr(col, attr_bool2)
        col += 1

        attr_float2 = grd.GridCellAttr()
        attr_float2.SetReadOnly(False)
        attr_float2.SetEditor(grd.GridCellFloatEditor(precision=1))
        attr_float2.SetRenderer(grd.GridCellFloatRenderer(precision=1))
        self.grd_table.SetColSize(col, 60)
        self.grd_table.SetColLabelValue(col, u"Decl.")
        self.grd_table.SetColAttr(col, attr_float2)
        col += 1

        attr_nref = grd.GridCellAttr()
        attr_nref.SetReadOnly(False)
        attr_nref.SetEditor(grd.GridCellChoiceEditor(choices=BearingNorthRefs.get_str_list()))
        attr_nref.SetRenderer(grd.GridCellStringRenderer())
        self.grd_table.SetColSize(col, 120)
        self.grd_table.SetColLabelValue(col, u"N. Ref.")
        self.grd_table.SetColAttr(col, attr_nref)
        col += 1

        attr_float3 = grd.GridCellAttr()
        attr_float3.SetReadOnly(False)
        attr_float3.SetEditor(grd.GridCellFloatEditor(precision=1))
        attr_float3.SetRenderer(grd.GridCellFloatRenderer(precision=1))
        self.grd_table.SetColSize(col, 60)
        self.grd_table.SetColLabelValue(col, u"Ref.\r\nMark. Dir.")
        self.grd_table.SetColAttr(col, attr_float3)
        col += 1

        attr_float4 = grd.GridCellAttr()
        attr_float4.SetReadOnly(False)
        attr_float4.SetEditor(grd.GridCellFloatEditor(precision=1))
        attr_float4.SetRenderer(grd.GridCellFloatRenderer(precision=1))
        self.grd_table.SetColSize(col, 60)
        self.grd_table.SetColLabelValue(col, u"Azim. Corr.")
        self.grd_table.SetColAttr(col, attr_float4)
        col += 1

        # load device site settings list:
        self.device_site_settings = {}
        self._load_site_settings()

    def _load_site_settings(self):
        """Loads the site depending device settings from file"""
        self.device_site_settings = fh.load_from_json(fh.DEVICE_SITE_SETTINGS_NAME)

    def _get_dev_site_setting(self, dev_id: str):
        """Tries to get site depending setting for the given device serial number.

        :param dev_id: The device identifier (part number + serial number) of the device.
        :return: Device settings from file or default data.
        :rtype: dict
        """
        retval01 = self.device_site_settings.get(dev_id)
        if not retval01:
            retval01 = {"dev_id": dev_id,
                            "Comment": "", "Unit": str(LevelUnits.dBm), "Atten": 0.0, "AutoAtten": True,
                            "Decl": 0.0, "NRef": str(BearingNorthRefs.REFERENCE_MARK_DIRECTION),
                            "RefMarkDir": 0.0, "AzimCorr": 0.0}
        return retval01

    def _save_site_settings(self):
        """Saves the site depending device settings from file"""
        fh.save_as_json(self.device_site_settings, fh.DEVICE_SITE_SETTINGS_NAME)

    def update_progress(self, evt: ProgressEvent):
        """Receives data from thread and updates progressbar and message text

        :param evt: Event of type ProgressEvent.
        """
        # Handle progress
        self.update_progress_bar(evt, self.pgb_progress)

        # Handle message
        if evt.msg is not None:
            self.lbl_status.SetLabelText(evt.msg)

        # Handle data:
        if evt.data is not None:
            data = evt.data
            pn_number = str(data["PN"])
            sn_number = str(data["SN"])
            dev_id = "{}_{}".format(pn_number, sn_number)

            self.grd_table.AppendRows(1)
            row_index = self.grd_table.GetNumberRows() - 1
            self.grd_table.SetCellValue(row_index, 0, "1")
            self.grd_table.SetCellValue(row_index, 1, str(data["IP"]))
            self.grd_table.SetCellValue(row_index, 2, str(data["MAC"]))
            self.grd_table.SetCellValue(row_index, 3, pn_number)
            self.grd_table.SetCellValue(row_index, 4, sn_number)
            self.grd_table.SetCellValue(row_index, 5, data["IDN"])

            site_settings = self._get_dev_site_setting(dev_id)
            self.grd_table.SetCellValue(row_index, 6, str(site_settings["Comment"]))
            self.grd_table.SetCellValue(row_index, 7, str(site_settings["Unit"]))
            self.grd_table.SetCellValue(row_index, 8, str(site_settings["Atten"]))
            self.grd_table.SetCellValue(row_index, 9, tb.bool2_grid_str(site_settings["AutoAtten"]))
            self.grd_table.SetCellValue(row_index, 10, str(site_settings["Decl"]))
            self.grd_table.SetCellValue(row_index, 11, str(site_settings["NRef"]))
            self.grd_table.SetCellValue(row_index, 12, str(site_settings["RefMarkDir"]))
            self.grd_table.SetCellValue(row_index, 13, str(site_settings["AzimCorr"]))

        # Handle buttons:
        if evt.btn_style:
            self.update_btn_style(evt.btn_style)

        # Handle icons:
        if evt.icon_style:
            pass  # self.update_icon_style(self.bmp_icon, evt.icon_style)

        # Handle help text:
        if evt.help_text:
            self.help_text = evt.help_text

        self.Layout()
        self.Refresh()
        self.Update()

    def on_ok_button_click(self, event):
        self.save_devices()
        event.Skip()

    def save_devices(self):
        return_id = wx.ID_OK
        devices = {}

        try:
            self.grd_table.SaveEditControlValue()
            self.Update()

            # Evaluate selected devices from list
            num_rows = self.grd_table.GetNumberRows()
            for row_index in range(num_rows):
                if "1" in self.grd_table.GetCellValue(row_index, 0):
                    device = {}
                    ip = str(self.grd_table.GetCellValue(row_index, 1))
                    mac = str(self.grd_table.GetCellValue(row_index, 2))
                    pn_number = str(self.grd_table.GetCellValue(row_index, 3))
                    sn_number = str(self.grd_table.GetCellValue(row_index, 4))
                    dev_id = "{}_{}".format(pn_number, sn_number)
                    device["IP"] = ip
                    device["MAC"] = mac
                    device["PN"] = pn_number
                    device["SN"] = sn_number
                    device["IDN"] = str(self.grd_table.GetCellValue(row_index, 5))
                    devices[ip] = device

                    site_settings = {}
                    site_settings["PN"] = pn_number
                    site_settings["SN"] = sn_number
                    site_settings["Comment"] = str(self.grd_table.GetCellValue(row_index, 6))
                    site_settings["Unit"] = str(self.grd_table.GetCellValue(row_index, 7))
                    site_settings["Atten"] = float(self.grd_table.GetCellValue(row_index, 8))
                    site_settings["AutoAtten"] = ("1" in self.grd_table.GetCellValue(row_index, 9))
                    site_settings["Decl"] = float(self.grd_table.GetCellValue(row_index, 10))
                    site_settings["NRef"] = str(self.grd_table.GetCellValue(row_index, 11))
                    site_settings["RefMarkDir"] = float(self.grd_table.GetCellValue(row_index, 12))
                    site_settings["AzimCorr"] = float(self.grd_table.GetCellValue(row_index, 13))
                    self.device_site_settings[dev_id] = site_settings

            # Write selected devices to file
            if devices:
                fh.save_as_json(devices, fh.DEVICE_LIST_NAME)
                self._save_site_settings()
        except:
            return_id = wx.ID_CANCEL


# This is just a python module containing helper classes/functions:
if __name__ == '__main__':
    pass
